@@ -15,6 +15,8 @@ import os
 import re
 import textwrap
 from babel._compat import izip, imap
+import pytz as _pytz
+from babel import localtime
 
 missing = object()
 
@@ -41,6 +43,8 @@ def distinct(iterable):
 # Regexp to match python magic encoding line
 PYTHON_MAGIC_COMMENT_re = re.compile(
     br'[ \t\f]* \# .* coding[=:][ \t]*([-\w.]+)', re.VERBOSE)
+
+
 def parse_encoding(fp):
     """Deduce the encoding of a source file from magic comment.
 
@@ -65,7 +69,7 @@ def parse_encoding(fp):
             try:
                 import parser
                 parser.suite(line1.decode('latin-1'))
-            except (ImportError, SyntaxError):
+            except (ImportError, SyntaxError, UnicodeEncodeError):
                 # Either it's a real syntax error, in which case the source is
                 # not valid python source, or line2 is a continuation of line1,
                 # in which case we don't want to scan line2 for a magic
@@ -90,6 +94,30 @@ def parse_encoding(fp):
     finally:
         fp.seek(pos)
 
+
+PYTHON_FUTURE_IMPORT_re = re.compile(
+    r'from\s+__future__\s+import\s+\(*(.+)\)*')
+
+
+def parse_future_flags(fp, encoding='latin-1'):
+    """Parse the compiler flags by :mod:`__future__` from the given Python
+    code.
+    """
+    import __future__
+    pos = fp.tell()
+    fp.seek(0)
+    flags = 0
+    try:
+        body = fp.read().decode(encoding)
+        for m in PYTHON_FUTURE_IMPORT_re.finditer(body):
+            names = [x.strip() for x in m.group(1).split(',')]
+            for name in names:
+                flags |= getattr(__future__, name).compiler_flag
+    finally:
+        fp.seek(pos)
+    return flags
+
+
 def pathmatch(pattern, filename):
     """Extended pathname pattern matching.
 
@@ -119,12 +147,12 @@ def pathmatch(pattern, filename):
     :param filename: the path name of the file to match against
     """
     symbols = {
-        '?':   '[^/]',
-        '?/':  '[^/]/',
-        '*':   '[^/]+',
-        '*/':  '[^/]+/',
+        '?': '[^/]',
+        '?/': '[^/]/',
+        '*': '[^/]+',
+        '*/': '[^/]+/',
         '**/': '(?:.+/)*?',
-        '**':  '(?:.+/)*?[^/]+',
+        '**': '(?:.+/)*?[^/]+',
     }
     buf = []
     for idx, part in enumerate(re.split('([?*]+/?)', pattern)):
@@ -165,6 +193,7 @@ class odict(dict):
 
     :see: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/107747
     """
+
     def __init__(self, data=None):
         dict.__init__(self, data or {})
         self._keys = list(dict.keys(self))
@@ -216,7 +245,7 @@ class odict(dict):
         self._keys.remove(key)
         return dict.popitem(key)
 
-    def setdefault(self, key, failobj = None):
+    def setdefault(self, key, failobj=None):
         dict.setdefault(self, key, failobj)
         if key not in self._keys:
             self._keys.append(key)
@@ -257,9 +286,6 @@ class FixedOffsetTimezone(tzinfo):
         return ZERO
 
 
-import pytz as _pytz
-from babel import localtime
-
 # Export the localtime functionality here because that's
 # where it was in the past.
 UTC = _pytz.utc
